@@ -6,10 +6,10 @@
  * the above copyright notice appear in all copies and that both that
  * copyright notice and this permission notice appear in supporting
  * documentation.
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -17,7 +17,7 @@
  * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
  * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  * OTHER DEALINGS IN THE SOFTWARE.
- * 
+ *
  * Except as contained in this notice, the name of The Open Group shall
  * not be used in advertising or otherwise to promote the sale, use or
  * other dealings in this Software without prior written authorization
@@ -25,7 +25,8 @@
  *
  */
 
-/* Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
+/*
+ * Copyright (c) 2005, Oracle and/or its affiliates.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -60,106 +61,116 @@
  *
  * usage: sessreg [ -w <wtmp-file> ] [ -u <utmp-file> ]
  *		  [ -l <line> ]
- *		  [ -L <lastlog-file> ]		      / #ifndef NO_LASTLOG
+ *		  [ -L <lastlog-file> ]		      / #ifdef USE_LASTLOG
  *		  [ -h <host-name> ]				/ BSD only
  *		  [ -s <slot-number> ] [ -x Xservers-file ]	/ BSD only
  *		  [ -t <ttys-file> ]				/ BSD only
- *	          [ -a ] [ -d ] user-name
+ *		  [ -a ] [ -d ] user-name
  *
  * one of -a or -d must be specified
  */
 
 #include "sessreg.h"
 
-# include	<X11/Xos.h>
-# include	<X11/Xfuncs.h>
-# include	<stdio.h>
-# include	<stdlib.h>
-
-#if defined(__SVR4) || defined(SVR4) || defined(linux) || defined(__GLIBC__)
-# define SYSV
-#endif
-
+#include <X11/Xos.h>
+#include <X11/Xfuncs.h>
+#include <X11/Xfuncproto.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <time.h>
-#define Time_t time_t
 
 #ifdef USE_UTMP
 static void set_utmp (struct utmp *u, char *line, char *user, char *host,
-		      Time_t date, int addp);
+		      time_t date, int addp);
 #endif
 
 #ifdef USE_UTMPX
 static void set_utmpx (struct utmpx *u, const char *line, const char *user,
-		       const char *host, Time_t date, int addp);
+		       const char *host, time_t date, int addp);
 #endif
 
 static int wflag, uflag, lflag;
-static char *wtmp_file, *utmp_file, *line;
+static const char *wtmp_file, *utmp_file;
 #ifdef USE_UTMPX
 #ifdef HAVE_UPDWTMPX
-static char *wtmpx_file = NULL;
+static const char *wtmpx_file = NULL;
 #endif
 #ifdef HAVE_UTMPXNAME
-static char *utmpx_file = NULL;
+static const char *utmpx_file = NULL;
 #endif
 #endif
+
 static int utmp_none, wtmp_none;
 /*
  * BSD specific variables.  To make life much easier for Xstartup/Xreset
  * maintainers, these arguments are accepted but ignored for sysV
  */
-static int hflag, sflag, xflag, tflag;
+static int hflag, xflag, tflag;
 static char *host_name = NULL;
-#ifdef USE_UTMP
+#if defined(USE_UTMP) && !defined(HAVE_PUTUTLINE)
+static int sflag;
 static int slot_number;
 #endif
 static char *xservers_file, *ttys_file;
 static char *user_name;
 static int aflag, dflag;
-#ifndef NO_LASTLOG
-static char *llog_file;
+#ifdef USE_LASTLOG
+static const char *llog_file;
 static int llog_none, Lflag;
 #endif
 
 static char *program_name;
 
-#ifndef SYSV
+#if defined(USE_UTMP) && !defined(HAVE_PUTUTLINE)
 static int findslot (char *line_name, char *host_name, int addp, int slot);
 static int Xslot (char *ttys_file, char *servers_file, char *tty_line,
 		  char *host_name, int addp);
 #endif
 
-static int
+static void _X_NORETURN _X_COLD
 usage (int x)
 {
-	if (x) {
-		fprintf (stderr, "%s: usage %s {-a -d} [-w wtmp-file] [-u utmp-file]", program_name, program_name);
-#ifndef NO_LASTLOG
-		fprintf (stderr, " [-L lastlog-file]");
+	fprintf (stderr,
+		 "%s: usage %s {-a -d} [-w wtmp-file] [-u utmp-file]"
+#ifdef USE_LASTLOG
+		 " [-L lastlog-file]"
 #endif
-		fprintf (stderr, "\n");
-		fprintf (stderr, "             [-t ttys-file] [-l line-name] [-h host-name]\n");
-		fprintf (stderr, "             [-s slot-number] [-x servers-file] user-name\n");
-		exit (1);
-	}
-	return x;
+		 "\n"
+		 "             [-t ttys-file] [-l line-name] [-h host-name] [-V]\n"
+		 "             [-s slot-number] [-x servers-file] user-name\n",
+		 program_name, program_name);
+	exit (x);
 }
 
 static char *
 getstring (char ***avp, int *flagp)
 {
 	char	**a = *avp;
+	char	*flag = *a;
 
-	usage ((*flagp)++);
+	if (*flagp != 0) {
+		fprintf (stderr, "%s: cannot give more than one -%s option\n",
+			 program_name, flag);
+		usage (1);
+	}
+	*flagp = 1;
+	/* if the argument is given immediately following the flag,
+	   i.e. "sessreg -hfoo ...", not "sessreg -h foo ...",
+	   then return the rest of the string as the argument value */
 	if (*++*a)
 		return *a;
+	/* else use the next pointer in the argv list as the argument value */
 	++a;
-	usage (!*a);
+	if (!*a) {
+		fprintf (stderr, "%s: -%s requires an argument\n",
+			 program_name, flag);
+		usage (1);
+	}
 	*avp = a;
 	return *a;
 }
 
-#ifndef SYSV
+#if defined(USE_UTMP) && !defined(HAVE_PUTUTLINE)
 static int
 syserr (int x, const char *s)
 {
@@ -181,23 +192,38 @@ sysnerr (int x, const char *s)
 	return x;
 }
 
+/*
+ * While this looks like it could be replaced with strlcpy() on platforms
+ * that have it, we're sticking with strncpy() so that we zero out the
+ * whole buffer to avoid writing garbage to the fixed length fields in the
+ * utmp/wtmp files, since strlcpy() does not write past the \0 terminator.
+ */
+static void
+safe_strncpy(char *dest, const char *src, size_t n)
+{
+    if (n > 0) {
+        strncpy(dest, src, n - 1);
+        dest[n - 1] = '\0';
+    }
+}
+
 int
 main (int argc, char **argv)
 {
-#if defined(USE_UTMP) && !defined(SYSV)
+#if defined(USE_UTMP) && !defined(HAVE_PUTUTLINE)
 	int		utmp;
 #endif
-	char		*line_tmp;
-#ifndef USE_UTMPX	
+#ifndef USE_UTMPX
 	int		wtmp;
-#endif	
-	Time_t		current_time;
+#endif
+	time_t		current_time;
 #ifdef USE_UTMP
 	struct utmp	utmp_entry;
 #endif
 #ifdef USE_UTMPX
 	struct utmpx	utmpx_entry;
 #endif
+	char *		line = NULL;
 
 	program_name = argv[0];
 	while (*++argv && **argv == '-') {
@@ -206,13 +232,21 @@ main (int argc, char **argv)
 			wtmp_file = getstring (&argv, &wflag);
 			if (!strcmp (wtmp_file, "none"))
 				wtmp_none = 1;
+#if defined(USE_UTMPX) && defined(HAVE_UPDWTMPX)
+			else
+				wtmpx_file = wtmp_file;
+#endif
 			break;
 		case 'u':
 			utmp_file = getstring (&argv, &uflag);
 			if (!strcmp (utmp_file, "none"))
 				utmp_none = 1;
+#if defined(USE_UTMPX) && defined(HAVE_UTMPXNAME)
+			else
+				utmpx_file = utmp_file;
+#endif
 			break;
-#ifndef NO_LASTLOG
+#ifdef USE_LASTLOG
 		case 'L':
 			llog_file = getstring (&argv, &Lflag);
 			if (!strcmp (llog_file, "none"))
@@ -229,7 +263,7 @@ main (int argc, char **argv)
 			host_name = getstring (&argv, &hflag);
 			break;
 		case 's':
-#ifdef USE_UTMP
+#if defined(USE_UTMP) && !defined(HAVE_PUTUTLINE)
 			slot_number = atoi (getstring (&argv, &sflag));
 #endif
 			break;
@@ -242,18 +276,49 @@ main (int argc, char **argv)
 		case 'd':
 			dflag++;
 			break;
+		case 'V':
+			puts(PACKAGE_STRING);
+			exit (0);
+                case '-':
+			if (strcmp(*argv, "-help") == 0) {
+				usage (0);
+			}
+			if (strcmp(*argv, "-version") == 0) {
+				puts (PACKAGE_STRING);
+				exit (0);
+			}
+			/* fallthrough */
 		default:
+			fprintf (stderr, "%s: unrecognized option '-%s'\n",
+				 program_name, argv[0]);
 			usage (1);
 		}
 	}
-	usage (!(user_name = *argv++));
-	usage (*argv != NULL);
+	user_name = *argv++;
+	if (user_name == NULL) {
+		fprintf (stderr, "%s: missing required user-name argument\n",
+			 program_name);
+		usage (1);
+	}
+	if (*argv != NULL) {
+		fprintf (stderr, "%s: unrecognized argument '%s'\n",
+			 program_name, argv[0]);
+		usage (1);
+	}
 	/*
 	 * complain if neither aflag nor dflag are set,
 	 * or if both are set.
 	 */
-	usage (!(aflag ^ dflag));
-	usage (xflag && !lflag);
+	if (!(aflag ^ dflag)) {
+		fprintf (stderr, "%s: must specify exactly one of -a or -d\n",
+			 program_name);
+		usage (1);
+	}
+	if (xflag && !lflag) {
+		fprintf (stderr, "%s: must specify -l when -x is used\n",
+			 program_name);
+		usage (1);
+	}
 	/* set up default file names */
 	if (!wflag) {
 		wtmp_file = WTMP_FILE;
@@ -261,21 +326,17 @@ main (int argc, char **argv)
 		wtmpx_file = WTMPX_FILE;
 #endif
 	}
-#ifndef NO_UTMP
 	if (!uflag) {
 		utmp_file = UTMP_FILE;
 #if defined(USE_UTMPX) && defined(HAVE_UTMPXNAME)
 		utmpx_file = UTMPX_FILE;
 #endif
 	}
-#else
-	utmp_none = 1;
-#endif
-#ifndef NO_LASTLOG
+#ifdef USE_LASTLOG
 	if (!Lflag)
 		llog_file = LLOG_FILE;
 #endif
-#if defined(USE_UTMP) && !defined(SYSV) && !defined(linux) && !defined(__QNX__)
+#if defined(USE_UTMP) && !defined(HAVE_PUTUTLINE)
 	if (!tflag)
 		ttys_file = TTYS_FILE;
 	if (!sflag && !utmp_none) {
@@ -286,12 +347,9 @@ main (int argc, char **argv)
 	}
 #endif
 	if (!lflag) {
-		sysnerr ((line_tmp = ttyname (0)) != NULL, "ttyname");
-		line = strrchr(line_tmp, '/');
-		if (line)
-			line = line + 1;
-		else
-			line = line_tmp;
+		sysnerr ((line = ttyname (0)) != NULL, "ttyname");
+		if (strncmp(line, "/dev/", 5) == 0)
+			line += 5;
 	}
 	time (&current_time);
 #ifdef USE_UTMP
@@ -303,27 +361,27 @@ main (int argc, char **argv)
 	   UtmpxIdOpen to work */
 # ifdef HAVE_UTMPXNAME
 	if (utmpx_file != NULL) {
-	        utmpxname (utmpx_file);
+		utmpxname (utmpx_file);
 	}
 # endif
 	set_utmpx (&utmpx_entry, line, user_name,
 		   host_name, current_time, aflag);
-#endif	
+#endif
 
 	if (!utmp_none) {
 #ifdef USE_UTMPX
-# ifdef HAVE_UTMPX_NAME
-	    if (utmpx_file != NULL)
+# ifdef HAVE_UTMPXNAME
+		if (utmpx_file != NULL)
 # endif
-	    {
-		setutxent ();
-		(void) getutxid (&utmpx_entry);
-		pututxline (&utmpx_entry);
-		endutxent ();
-	    }
+		{
+			setutxent ();
+			(void) getutxid (&utmpx_entry);
+			pututxline (&utmpx_entry);
+			endutxent ();
+		}
 #endif
 #ifdef USE_UTMP
-# ifdef SYSV
+# ifdef HAVE_PUTUTLINE
 		utmpname (utmp_file);
 		setutent ();
 		(void) getutid (&utmp_entry);
@@ -332,9 +390,9 @@ main (int argc, char **argv)
 # else
 		utmp = open (utmp_file, O_RDWR);
 		if (utmp != -1) {
-			syserr ((int) lseek (utmp, (long) slot_number * sizeof (struct utmp), 0), "lseek");
+			syserr ((int) lseek (utmp, (off_t) slot_number * sizeof (struct utmp), 0), "lseek");
 			sysnerr (write (utmp, (char *) &utmp_entry, sizeof (utmp_entry))
-				        == sizeof (utmp_entry), "write utmp entry");
+					== sizeof (utmp_entry), "write utmp entry");
 			close (utmp);
 		}
 # endif
@@ -351,33 +409,33 @@ main (int argc, char **argv)
 		wtmp = open (wtmp_file, O_WRONLY|O_APPEND);
 		if (wtmp != -1) {
 			sysnerr (write (wtmp, (char *) &utmp_entry, sizeof (utmp_entry))
-				        == sizeof (utmp_entry), "write wtmp entry");
+					== sizeof (utmp_entry), "write wtmp entry");
 			close (wtmp);
 		}
-#endif		
+#endif
 	}
-#ifndef NO_LASTLOG
+#ifdef USE_LASTLOG
 	if (aflag && !llog_none) {
-	        int llog;
-	        struct passwd *pwd = getpwnam(user_name);
+		int llog;
+		struct passwd *pwd = getpwnam(user_name);
 
-	        sysnerr( pwd != NULL, "get user id");
-	        llog = open (llog_file, O_RDWR);
+		sysnerr( pwd != NULL, "get user id");
+		llog = open (llog_file, O_RDWR);
 
 		if (llog != -1) {
 			struct lastlog ll;
 
-			sysnerr (lseek(llog, (long) pwd->pw_uid*sizeof(ll), 0)
-				        != -1, "seeking lastlog entry");
-			bzero((char *)&ll, sizeof(ll));
+			sysnerr (lseek(llog, (off_t) (pwd->pw_uid*sizeof(ll)), 0)
+					!= -1, "seeking lastlog entry");
+			memset(&ll, 0, sizeof(ll));
 			ll.ll_time = current_time;
 			if (line)
-			 (void) strncpy (ll.ll_line, line, sizeof (ll.ll_line));
+			 safe_strncpy (ll.ll_line, line, sizeof (ll.ll_line));
 			if (host_name)
-			 (void) strncpy (ll.ll_host, host_name, sizeof (ll.ll_host));
+			 safe_strncpy (ll.ll_host, host_name, sizeof (ll.ll_host));
 
 			sysnerr (write (llog, (char *) &ll, sizeof (ll))
-				        == sizeof (ll), "write lastlog entry");
+					== sizeof (ll), "write lastlog entry");
 			close (llog);
 		}
 	}
@@ -391,20 +449,20 @@ main (int argc, char **argv)
 
 #ifdef USE_UTMP
 static void
-set_utmp (struct utmp *u, char *line, char *user, char *host, Time_t date, int addp)
+set_utmp (struct utmp *u, char *line, char *user, char *host, time_t date, int addp)
 {
-	bzero (u, sizeof (*u));
+	memset (u, 0, sizeof (*u));
 	if (line)
-		(void) strncpy (u->ut_line, line, sizeof (u->ut_line));
+		safe_strncpy (u->ut_line, line, sizeof (u->ut_line));
 	else
-		bzero (u->ut_line, sizeof (u->ut_line));
+		memset (u->ut_line, 0, sizeof (u->ut_line));
 	if (addp && user)
-		(void) strncpy (u->ut_name, user, sizeof (u->ut_name));
+		safe_strncpy (u->ut_name, user, sizeof (u->ut_name));
 	else
-		bzero (u->ut_name, sizeof (u->ut_name));
-#ifdef SYSV
+		memset (u->ut_name, 0, sizeof (u->ut_name));
+#ifdef HAVE_STRUCT_UTMP_UT_ID
 	if (line) {
-		int	i;
+		size_t	i;
 		/*
 		 * this is a bit crufty, but
 		 * follows the apparent conventions in
@@ -417,22 +475,27 @@ set_utmp (struct utmp *u, char *line, char *user, char *host, Time_t date, int a
 			i -= sizeof (u->ut_id);
 		else
 			i = 0;
-		(void) strncpy (u->ut_id, line + i, sizeof (u->ut_id));
+		safe_strncpy (u->ut_id, line + i, sizeof (u->ut_id));
 	} else
-		bzero (u->ut_id, sizeof (u->ut_id));
-	if (addp) {
-		u->ut_pid = getppid ();
-		u->ut_type = USER_PROCESS;
-	} else {
-		u->ut_pid = 0;
-		u->ut_type = DEAD_PROCESS;
-	}
+		memset (u->ut_id, 0, sizeof (u->ut_id));
 #endif
-#if (!defined(SYSV) && !defined(__QNX__)) || defined(linux)
-	if (addp && host)
-		(void) strncpy (u->ut_host, host, sizeof (u->ut_host));
+#ifdef HAVE_STRUCT_UTMP_UT_PID
+	if (addp)
+		u->ut_pid = getppid ();
 	else
-		bzero (u->ut_host, sizeof (u->ut_host));
+		u->ut_pid = 0;
+#endif
+#ifdef HAVE_STRUCT_UTMP_UT_TYPE
+	if (addp)
+		u->ut_type = USER_PROCESS;
+	else
+		u->ut_type = DEAD_PROCESS;
+#endif
+#ifdef HAVE_STRUCT_UTMP_UT_HOST
+	if (addp && host)
+		safe_strncpy (u->ut_host, host, sizeof (u->ut_host));
+	else
+		memset (u->ut_host, 0, sizeof (u->ut_host));
 #endif
 	u->ut_time = date;
 }
@@ -446,110 +509,111 @@ UtmpxIdOpen( char *utmpId )
 	int    status = 1;	/* return code                             */
 
 	setutxent();
- 
+
 	while ( (u = getutxent()) != NULL ) {
-		
+
 		if ( (strncmp(u->ut_id, utmpId, 4) == 0 ) &&
 		     u->ut_type != DEAD_PROCESS ) {
-			
+
 			status = 0;
 			break;
 		}
 	}
- 
+
 	endutxent();
 	return (status);
 }
 
 static void
 set_utmpx (struct utmpx *u, const char *line, const char *user,
-	   const char *host, Time_t date, int addp)
+	   const char *host, time_t date, int addp)
 {
 	static const char letters[] =
 	       "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
-        if (line)
+	memset (u, 0, sizeof (*u));
+	if (line)
 	{
-                if(strcmp(line, ":0") == 0)
-                        (void) strcpy(u->ut_line, "console");
-                else
-                        (void) strncpy (u->ut_line, line, sizeof (u->ut_line));
+		if(strcmp(line, ":0") == 0)
+			(void) strcpy(u->ut_line, "console");
+		else
+			safe_strncpy (u->ut_line, line, sizeof (u->ut_line));
 
-		strncpy(u->ut_host, line, sizeof(u->ut_host));
-#if HAVE_UTMPX_UT_SYSLEN
-		u->ut_syslen = strlen(line); 
+		safe_strncpy(u->ut_host, line, sizeof(u->ut_host));
+#ifdef HAVE_STRUCT_UTMPX_UT_SYSLEN
+		u->ut_syslen = strlen(line);
 #endif
 	}
-        else
-                bzero (u->ut_line, sizeof (u->ut_line));
-        if (addp && user)
-                (void) strncpy (u->ut_user, user, sizeof (u->ut_user));
-        else
-                bzero (u->ut_user, sizeof (u->ut_user));
+	else
+		memset (u->ut_line, 0, sizeof (u->ut_line));
+	if (addp && user)
+		safe_strncpy (u->ut_user, user, sizeof (u->ut_user));
+	else
+		memset (u->ut_user, 0, sizeof (u->ut_user));
 
-        if (line) {
-                int     i;
-                /*
-                 * this is a bit crufty, but
-                 * follows the apparent conventions in
-                 * the ttys file.  ut_id is only 4 bytes
-                 * long, and the last 4 bytes of the line
-                 * name are written into it, left justified.
-                 */
-                i = strlen (line);
-                if (i >= sizeof (u->ut_id))
-                        i -= sizeof (u->ut_id);
-                else
-                        i = 0;
-                (void) strncpy (u->ut_id, line + i, sizeof (u->ut_id));
+	if (line) {
+		size_t	i;
+		/*
+		 * this is a bit crufty, but
+		 * follows the apparent conventions in
+		 * the ttys file.  ut_id is only 4 bytes
+		 * long, and the last 4 bytes of the line
+		 * name are written into it, left justified.
+		 */
+		i = strlen (line);
+		if (i >= sizeof (u->ut_id))
+			i -= sizeof (u->ut_id);
+		else
+			i = 0;
+		safe_strncpy (u->ut_id, line + i, sizeof (u->ut_id));
 
 		/* make sure there is no entry using identical ut_id */
 		if (!UtmpxIdOpen(u->ut_id) && addp) {
-                        int limit = sizeof(letters) - 1;
-                        int t = 0;
+			int limit = sizeof(letters) - 1;
+			int t = 0;
 
-                        u->ut_id[1] = line[i];
-                        u->ut_id[2] = line[i+1];
-                        u->ut_id[3] = line[i+2];
-                        do {
-                                u->ut_id[0] = letters[t];
-                                t++;
-                        } while (!UtmpxIdOpen(u->ut_id) && (t < limit));
-                }
-                if (!addp && strstr(line, ":") != NULL) {
-                        struct utmpx *tmpu;             
+			u->ut_id[1] = line[i];
+			u->ut_id[2] = line[i+1];
+			u->ut_id[3] = line[i+2];
+			do {
+				u->ut_id[0] = letters[t];
+				t++;
+			} while (!UtmpxIdOpen(u->ut_id) && (t < limit));
+		}
+		if (!addp && strstr(line, ":") != NULL) {
+			struct utmpx *tmpu;
 
-                        while ( (tmpu = getutxent()) != NULL ) {
-                                if ( (strcmp(tmpu->ut_host, line) == 0 ) &&
-                                        tmpu->ut_type != DEAD_PROCESS ) {
-                                        strncpy(u->ut_id, tmpu->ut_id,
+			while ( (tmpu = getutxent()) != NULL ) {
+				if ( (strcmp(tmpu->ut_host, line) == 0 ) &&
+					tmpu->ut_type != DEAD_PROCESS ) {
+					strncpy(u->ut_id, tmpu->ut_id,
 						sizeof(u->ut_id));
-                                        break;
-                                }
-                        }
-                        endutxent();
-                }
-        } else
-                bzero (u->ut_id, sizeof (u->ut_id));
-	
-        if (addp) {
-                u->ut_pid = getppid ();
-                u->ut_type = USER_PROCESS;
-        } else {
-                u->ut_pid = 0;
-                u->ut_type = DEAD_PROCESS;
-        }
+					break;
+				}
+			}
+			endutxent();
+		}
+	} else
+		memset (u->ut_id, 0, sizeof (u->ut_id));
+
+	if (addp) {
+		u->ut_pid = getppid ();
+		u->ut_type = USER_PROCESS;
+	} else {
+		u->ut_pid = 0;
+		u->ut_type = DEAD_PROCESS;
+	}
 	u->ut_tv.tv_sec = date;
 	u->ut_tv.tv_usec = 0;
 }
 #endif /* USE_UTMPX */
 
-#if defined(USE_UTMP) && !defined(SYSV)
+#if defined(USE_UTMP) && !defined(HAVE_PUTUTLINE)
 /*
  * compute the slot-number for an X display.  This is computed
  * by counting the lines in /etc/ttys and adding the line-number
  * that the display appears on in Xservers.  This is a poor
- * design, but is limited by the non-existant interface to utmp.
+ * design, but is limited by the non-existent interface to utmp.
  * If host_name is non-NULL, assume it contains the display name,
  * otherwise use the tty_line argument (i.e., the tty name).
  */
@@ -572,9 +636,9 @@ Xslot (char *ttys_file, char *servers_file, char *tty_line, char *host_name,
 	strncpy(disp_name, host_name ? host_name : tty_line, sizeof(disp_name)-1);
 	pos = strrchr(disp_name, ':');
 	if (pos) {
-	    pos = strchr(pos, '.');
-	    if (pos)
-		*pos = '\0';
+		pos = strchr(pos, '.');
+		if (pos)
+			*pos = '\0';
 	}
 	sysnerr ((int)(long)(ttys = fopen (ttys_file, "r")), ttys_file);
 	while ((c = getc (ttys)) != EOF)
@@ -604,7 +668,7 @@ Xslot (char *ttys_file, char *servers_file, char *tty_line, char *host_name,
 			column0 = 1;
 	}
 	/*
-	 * display not found in Xservers file - allocate utmp entry dinamically
+	 * display not found in Xservers file - allocate utmp entry dynamically
 	 */
 	return findslot (tty_line, host_name, addp, slot);
 }
@@ -628,20 +692,20 @@ findslot (char *line_name, char *host_name, int addp, int slot)
 	 * first, try to locate a previous entry for this display
 	 * also record location of a free slots in case we need a new one
 	 */
-	syserr ((int) lseek (utmp, (long) slot * sizeof (struct utmp), 0), "lseek");
+	syserr ((int) lseek (utmp, (off_t) slot * sizeof (struct utmp), 0), "lseek");
 
 	if (!host_name)
 		host_name = "";
 
 	while (read (utmp, (char *) &entry, sizeof (entry)) == sizeof (entry)) {
 		if (strncmp(entry.ut_line, line_name,
-			sizeof(entry.ut_line)) == 0 
-#ifndef __QNX__
-                    &&
+			sizeof(entry.ut_line)) == 0
+#ifdef HAVE_STRUCT_UTMP_UT_HOST
+		    &&
 		    strncmp(entry.ut_host, host_name,
 			sizeof(entry.ut_host)) == 0
 #endif
-                   ) {
+		   ) {
 			found = 1;
 			break;
 		}
